## Introduction
The SGX/TDX Attestation Appraisal functionality is part of DCAP Attestation. Several APIs and one tool are provided for the appraisal functionality:

### Appraisal APIs:
```bash
/**
 * Get quote verification result token.
 *
 * @param p_quote[IN] - Pointer to SGX Quote.
 * @param quote_size[IN] - Size of the buffer pointed to by p_quote (in bytes).
 * @param p_quote_collateral[IN] - The parameter is optional. This is a pointer to the Quote Certification Collateral provided by the caller.
 * @param p_qve_report_info[IN/OUT] - This parameter can be used in 2 ways.
 *        If p_qve_report_info is NOT NULL, the API will use Intel QvE to perform quote verification, and QvE will generate a report using the target_info in sgx_ql_qe_report_info_t structure.
 *        if p_qve_report_info is NULL, the API will use QVL library to perform quote verification, note that the results can not be cryptographically authenticated in this mode.
 * @param p_user_data[IN] - User data.
 * @param p_verification_result_token_buffer_size[OUT] - Size of the buffer pointed to by verification_result_token (in bytes).
 * @param p_verification_result_token[OUT] - Pointer to the verification_result_token.
 *
 * @return Status code of the operation, one of:
 *      - SGX_QL_SUCCESS
 *      - SGX_QL_ERROR_OUT_OF_MEMORY
 *      - SGX_QL_ERROR_UNEXPECTED
 **/
quote3_error_t  tee_verify_quote_qvt(
    const uint8_t *p_quote,
    uint32_t quote_size,
    const sgx_ql_qve_collateral_t *p_quote_collateral,
    sgx_ql_qe_report_info_t *p_qve_report_info,
    const uint8_t *p_user_data,
    uint32_t *p_verification_result_token_buffer_size,
    uint8_t **p_verification_result_token)

/**
 * Free quote verification result token buffer, which returned by `tee_verify_quote_qvt`
 *
 * @param p_verification_result_token[IN] - Pointer to verification result token
 * @param p_verification_result_token_buffer_size[IN] - Pointer to verification result token size
 *
 * @return Status code of the operation, one of:
 *      - SGX_QL_SUCCESS
 *      - SGX_QL_ERROR_INVALID_PARAMETER
 **/
quote3_error_t tee_free_verify_quote_qvt(
    uint8_t *p_verification_result_token,
    uint32_t *p_verification_result_token_buffer_size) 

/**
 * Appraise a Verification Result JWT against one or more Quote Appraisal Policies
 *
 * @param p_verification_result_token[IN] - Points to a null-terminated string containing the input Verification Result JWT.
 * @param p_qaps[IN] - Points to an array of pointers, with each pointer pointing to a buffer holding a quote appraisal policy JWT token. 
 *                     Each token is a null-terminated string holding a JWT.
 * @param qaps_count[IN] - The number of pointers in the p_qaps array.
 * @param appraisal_check_date[IN] - -	User input, used by the appraisal engine as its “current time” for expiration dates check.
 * @param p_qae_report_info[IN, OUT] - The parameter is optional.
 * @param p_appraisal_result_token_buffer_size[OUT] - Points to hold the size of the p_appraisal_result_token buffer.
 * @param p_appraisal_result_token[OUT] - Points to the output Appraisal result JWT.
 *
 * @return Status code of the operation. SGX_QL_SUCCESS or failure as defined in sgx_ql_lib_common.h
 **/
quote3_error_t tee_appraise_verification_token(
    const uint8_t *p_verification_result_token,
    uint8_t **p_qaps,
    uint8_t qaps_count,
    const time_t appraisal_check_date,
    sgx_ql_qe_report_info_t *p_qae_report_info,
    uint32_t *p_appraisal_result_token_buffer_size,
    uint8_t **p_appraisal_result_token)

/**
 * Free the appraisal result token that allocated in the "tee_appraise_verification_token" API
 * @param p_appraisal_result_token[IN] - Points to the output Appraisal result JWT.
 *
 * @return Status code of the operation. SGX_QL_SUCCESS or failure as defined in sgx_ql_lib_common.h
**/
quote3_error_t tee_free_appraisal_token(uint8_t *p_appraisal_result_token)


/**
 * Check whether the input policies are used in the appraisal process by comparing the policies with the appraisal result
 *
 * @param p_appraisal_result_token[IN] - Points to the Appraisal result JWT that generated by the "tee_appraise_verification_token" API
 * @param p_policies[IN] - A structure that contains the target policies
 * @param result[OUT] - the authentication result
 *
 * @return Status code of the operation. SGX_QL_SUCCESS or failure as defined in sgx_ql_lib_common.h
**/
quote3_error_t tee_authenticate_appraisal_result(const uint8_t *p_appraisal_result_token, const tee_policy_bundle_t *p_policies, tee_policy_auth_result_t *result)
    
```

### TEE Appraisal Tool:
SGX/TDX Appraisal provides a tool named [tee_appraisal_tool](tee_appraisal_tool) that assists users in creating signed appraisal policies with JWT format for a tenant SGX enclave or a tenant TD. This tool provides several commands as below:
```bash
      $ tee_appraisal_tool <Command> <options> <files>
      Command:
          gen_payload:        Generate the policy payload file with Json format based on the input enclave or TDX Report.
          sign_policy:        Sign the input policy payload with the input EC key and generate the final policy file with JWT format.
          verify_policy:      Verify the JWT policy file.
      Options:
          -in                 Specify the input file path
          -key                Specify the key file. The key must be a PEM-formatted, 384-byte EC private key.
                              It is a required option for "sign_policy"
          -out                Speicify the output file path
          -v                  Enable showing the extra dump message for each command

      Example:
          tee_appraisal_tool gen_payload -in {enclave/TDReport} -out payload.json [-v]
          tee_appraisal_tool sign_policy -in payload.json -key ec_private.pem -out policy.jwt [-v]
          tee_appraisal_tool verify_policy -in policy.jwt [-v]

      Run "tee_appraisal_tool -help" to get this help and exit.
      Run "tee_appraisal_tool -version" to output version information and exit.
```
This tool could be also used to sign the platform policies if you want to create a platform policy by yourself. Under [tee_appraisal_tool/data](tee_appraisal_tool/data), there are a few platform policy payload templates available. You may modify the payload and sign it using the `sign_policy` command to create your own platform policy.



#### Detail steps:
- Prepare necessary materials. To generate an SGX enclave policy, you need to prepare the target tenant SGX enclave. To generate a TDX policy, you need to prepare a TD Report for the target TD. And you may also need to prepare one EC signing key with the following command:
```bash
    $ openssl ecparam -name secp384r1 -genkey --noout > ec_private.pem
 ```

- Generate the policy payload file (json format) with below command:
```bash 
    $ tee_appraisal_tool gen_payload -in {enclave/TDReport}  -out payload.json [-v]
```
Then you could edit the generated policy payload if necessary.

- Sign the generated payload and generate the JWT format policy with an EC private signing key:
```bash
    $ tee_appraisal_tool sign_policy -in payload.json -key ec_private.pem -out policy.jwt [-v]
```

- Verify a JWT policy with below command if you want to:
```bash
    $ tee_appraisal_tool verify_policy -in policy.jwt
```

Please refer to [QuoteAppraisalSample](../../SampleCode/QuoteAppraisalSample) for the whole work flow for SGX/TDX Appraisal.
